/*
 *  ASA Library
 *  Copyright (C) 2004-2008 Distributed Systems Architecture Research Group
 *  http://asa.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package gamf.addons.junittests;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.List;

import org.junit.BeforeClass;
import org.junit.Test;

import gamf.framework.GenericAutonomicManagementFramework;
import gamf.framework.exceptions.ConfigurationError;
import gamf.framework.exceptions.DoubleManagedFacetNameOccurenceException;
import gamf.framework.exceptions.DoubleEventTypeOccuranceException;
import gamf.framework.exceptions.NotRegisteredManagedFacetReference;
import gamf.interfaces.framework.IManagedFacet;
import gamf.interfaces.framework.IEventStore;
import gamf.interfaces.framework.IGAMF;
import gamf.interfaces.systemAdapter.IEventGenerator;
import uk.ac.standrews.cs.nds.eventModel.IEvent;

/**
 * The following tests are evaluating if an event generator can be added (and
 * configured) to the GAMF as well as the event generator's functionality
 * (creating and sending events).
 * 
 * @author markus
 * 
 */

public class EventGeneratorTests {

	static IGAMF junitManager;
	static IManagedFacet managedFacet;
	static String junitManagedFacetName1;
	String junitManagedFacetSimilarToName1;
	static IEventGenerator eventGenerator, eventGenerator2;

	/**
	 * Creates a manager, an managedFacet and adds two eventGenerators. The
	 * EventGenerator creates an event type register. Therefore both
	 * instantiations of the class hold a similar set of event types. The GAMF
	 * is configured to detect similar event types.
	 */

	@BeforeClass
	public static void initialize() {
		junitManager = new GenericAutonomicManagementFramework();
		junitManagedFacetName1 = JunitTestRegister.MANAGEDFACETNAME1;

		// we don't care about that exceptions in this test case
		try {
			managedFacet = junitManager
					.registerManagedFacet(junitManagedFacetName1);
		} catch (DoubleManagedFacetNameOccurenceException e) {
			e.printStackTrace();
		}

		eventGenerator = new DummyEventGenerator();

		eventGenerator2 = new DummyEventGenerator();

		junitManager.setDependencyDetection(true);

	}

	/**
	 * Evaluates if GAMF detects similar event types before an eventGenerator is
	 * added if configured so. A DoubleEventTypeOccuranceException gets thrown
	 * if dependencyDetection is set and two event generators are added
	 * containing identical event types. The test fails if no
	 * DoubleEventTypeOccuranceException is thrown.
	 * 
	 * 
	 * @throws DoubleEventTypeOccuranceException
	 * @throws NotRegisteredManagedFacetReference
	 */

	@Test(expected = DoubleEventTypeOccuranceException.class)
	public void similarEventTypes() throws DoubleEventTypeOccuranceException,
			NotRegisteredManagedFacetReference {

		junitManager.add(eventGenerator, managedFacet);

		// add an instance of the JunitEventGenerator with "the same set of
		// events " than in the previous instance

		IManagedFacet managedFacet2 = null;
		try {
			managedFacet2 = junitManager
					.registerManagedFacet("junit managedFacet 2");
		} catch (DoubleManagedFacetNameOccurenceException e) {

			e.printStackTrace();
		}
		junitManager.add(eventGenerator2, managedFacet2);
		fail("previous function should have caused an exception because it holds identicall event types than the already added event generator");
	}

	/**
	 * checks if checkDependency is set - The test performed by
	 * similarEventTypes() only returns the expected exception if
	 * checkDependency is set
	 * 
	 */

	@Test
	public void isCheckDependencySet() {

		assertTrue(junitManager.getDependencyDetection());
	}

	/**
	 * Evaluates if all generated events are received and stored by the manager.
	 * By inserting and extracting events in and from the manager. The test
	 * fails if not all of the events are received and stored. The events
	 * generated are given by the systemAdapterExample eventGenerator. The
	 * eventGenerator used for this test is the one which was registered in a
	 * previous test.
	 */

	@Test
	public void sendAndReceiveEvents() {

		// this calls are generating 4 events
		try {
			((DummyEventGenerator) eventGenerator).populateEventStore();
			((DummyEventGenerator) eventGenerator).populateEventStore();
		} catch (ConfigurationError e) {
			fail(e.getLocalizedMessage());
		}

		// accessing the event store like a system adapter program has to do
		IEventStore eventsStored = junitManager.getEventStore();

		List<IEvent> eventsOfType = eventsStored
				.getAllOfType(JunitTestRegister.EVENTGENERATOREVENTTYPE_1);
		Integer storedEvents = new Integer(eventsOfType.size());

		// eventHandling is executed twice - therefore 2 events of that type are
		// stored
		assertTrue(storedEvents.equals(2));

		// double check possiblility if the event types are the ones we are
		// looking for
		// Iterator<IEvent> temp = eventsOfType.iterator();
		//		
		// while(temp.hasNext()){
		// System.out.println(temp.next().getType());
		// assertTrue(temp.next().getType().equals(WebServerExampleConstantsRegistry.EVENTGENERATOREVENTTYPE_1));
		//			
		// }

	}

}
