/*
 *  ASA Library
 *  Copyright (C) 2004-2008 Distributed Systems Architecture Research Group
 *  http://asa.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package gamf.framework;

import gamf.framework.exceptions.DoubleEventTypeOccuranceException;
import gamf.framework.exceptions.DoubleManagedFacetNameOccurenceException;
import gamf.framework.exceptions.EventGenerationException;
import gamf.framework.exceptions.NoEffectorException;
import gamf.framework.exceptions.NotRegisteredManagedFacetReference;
import gamf.interfaces.framework.IEventStore;
import gamf.interfaces.framework.IEventTrigger;
import gamf.interfaces.framework.IGAMF;
import gamf.interfaces.framework.IManagedFacet;
import gamf.interfaces.framework.IMetricValueStore;
import gamf.interfaces.framework.IPeriodicalTrigger;
import gamf.interfaces.framework.ITrigger;
import gamf.interfaces.systemAdapter.IEffector;
import gamf.interfaces.systemAdapter.IEventGenerator;
import gamf.interfaces.systemAdapter.IMetricExtractor;
import gamf.interfaces.systemAdapter.IPolicyEvaluator;
import gamf.interfaces.systemAdapter.ISystemAdapter;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import uk.ac.standrews.cs.nds.eventModel.eventBus.EventBus;
import uk.ac.standrews.cs.nds.eventModel.eventBus.busInterfaces.IEventBus;

/**
 * 
 * Provides access to the GAMF and describes a generic autonomic manager.
 * <p>
 * System adapter components can be registered, configured and categorised
 * through the GenericAutonomicManager. Further to that it allows the
 * configuration of the framework's behaviour. When a system adapter component
 * is registered it must also be associated with an managedFacet, therefore a
 * managedFacet must be defined before a component is registered.
 * </p>
 * <p>
 * In the case managers are nested, a manager provides an event generator
 * interface for a higher level manager, which's functionality is not
 * implemented yet.
 * </p>
 * 
 * @author markus
 * 
 */
public class GenericAutonomicManagementFramework implements IGAMF {

	private static Set<IManagedFacet> managedFacetRegistry;

	private static IEventStore eventStore;

	private static IMetricValueStore metricResultStore;

	private static Set<String> eventTypesRegister;

	private boolean dependencyDetection = false;

	@SuppressWarnings("unused")
	private EventRecorder eventRecorder;

	private static IEventBus managerEventBus;

	public GenericAutonomicManagementFramework() {
		managedFacetRegistry = new HashSet<IManagedFacet>();
		eventStore = new EventStore();
		metricResultStore = new MetricResultStore();

		managerEventBus = new EventBus();
		eventTypesRegister = new HashSet<String>();

		eventRecorder = new EventRecorder(managerEventBus, this);

	}

	public IManagedFacet registerManagedFacet(String managedFacetName)
			throws DoubleManagedFacetNameOccurenceException {

		for (IManagedFacet managedFacet : managedFacetRegistry) {
			if (managedFacet.getManagedFacetName().equals(managedFacetName)) {
				throw new DoubleManagedFacetNameOccurenceException(
						managedFacetName + " is already in use.\n");
			}
		}

		IManagedFacet managedFacet = new ManagedFacet(managedFacetName);
		managedFacetRegistry.add(managedFacet);

		return managedFacet;

	}

	public void add(IEffector effector, IManagedFacet managedFacet)
			throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException {

		// TODO save information which policy access effector??
		registerManagedFacet(managedFacet, effector);
		if (effector instanceof IEventGenerator) {
			((IEventGenerator) effector)
					.setEventBus(managerEventBus);

		}

	}

	public void add(IMetricExtractor metricExtractor, IEventTrigger trigger,
			IManagedFacet managedFacet)
			throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException {
		// TODO check if all components are registered and started (thread)
		// successfully
		// bundle exceptions ?
		registerManagedFacet(managedFacet, metricExtractor);

		metricExtractor.setManagerReferece(this);
		// TODO consider if there shouldn't be a flag for "internal compoments
		// e.g. evaluators)

		ITrigger triggeredMetricEvaluator = new MetricExtractor(metricExtractor,
				metricResultStore, managerEventBus);
		triggeredMetricEvaluator.start();
		registerManagedFacet(managedFacet, triggeredMetricEvaluator);
		IEventTrigger customTrigger = trigger;
		customTrigger.setEvaluator(triggeredMetricEvaluator);
		customTrigger.setEventBus(managerEventBus);
		registerManagedFacet(managedFacet, customTrigger);

	}

	public void add(IMetricExtractor metric, int waitPeriod,
			IManagedFacet managedFacet)
			throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException {

		// TODO check if all components are registered and started (thread)
		// successfully
		// bundle exceptions ?
		registerManagedFacet(managedFacet, metric);

		metric.setManagerReferece(this);
		// TODO consider if there shouldn't be a flag for "internal compoments
		// e.g. evaluators)
		IPeriodicalTrigger metricEvaluator = new PeriodicalMetricExtractor(
				metric, waitPeriod, metricResultStore, managerEventBus);
		metricEvaluator.start();
		registerManagedFacet(managedFacet, metricEvaluator);

	}

	public void add(IMetricExtractor metric, String eventNameToBeTriggerendOn,
			IManagedFacet managedFacet)
			throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException {
		// TODO check if all components are registered and started (thread)
		// successfully
		// bundle exceptions ?
		registerManagedFacet(managedFacet, metric);

		metric.setManagerReferece(this);
		// TODO consider if there shouldn't be a flag for "internal compoments
		// e.g. evaluators)

		ITrigger triggeredMetricEvaluator = new MetricExtractor(metric,
				metricResultStore, managerEventBus);
		registerManagedFacet(managedFacet, triggeredMetricEvaluator);

		IEventTrigger trigger = new Trigger(eventNameToBeTriggerendOn,
				triggeredMetricEvaluator, managerEventBus);
		registerManagedFacet(managedFacet, trigger);

	}

	public void add(IMetricExtractor metric, IManagedFacet managedFacet)
			throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException {
		// TODO check if all components are registered and started (thread)
		// successfully
		// bundle exceptions ?
		registerManagedFacet(managedFacet, metric);

		metric.setManagerReferece(this);

	}

	private void addPolicy(IPolicyEvaluator policy, IManagedFacet managedFacet)
			throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException, NoEffectorException {
		registerManagedFacet(managedFacet, policy);
		policy.setManagerReferece(this);

		Iterator<IEffector> usedEffectors = policy.getEffectors();
		if (usedEffectors != null) {
			while (usedEffectors.hasNext()) {
				IEffector usedEffector = usedEffectors.next();
				if (!managedFacet.contains(usedEffector)) {
					throw new NoEffectorException(
							"This policy "
									+ policy
									+ " uses an effector which is not registred with the GAM");
				}
			}
		}

	}

	public void add(IPolicyEvaluator policy, String eventNameToBeTriggerendOn,
			IManagedFacet managedFacet)
			throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException, NoEffectorException {
		addPolicy(policy, managedFacet);
		ITrigger evaluator = new PolicyEvaluator(policy);
		registerManagedFacet(managedFacet, evaluator);

		IEventTrigger trigger = new Trigger(eventNameToBeTriggerendOn,
				evaluator, managerEventBus);
		registerManagedFacet(managedFacet, trigger);

	}

	public void add(IPolicyEvaluator policy, IEventTrigger trigger,
			IManagedFacet managedFacet)
			throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException {
		ITrigger evaluator = new PolicyEvaluator(policy);
		registerManagedFacet(managedFacet, evaluator);

		registerManagedFacet(managedFacet, evaluator);
		IEventTrigger customTrigger = trigger;
		customTrigger.setEvaluator(evaluator);
		customTrigger.setEventBus(managerEventBus);
		registerManagedFacet(managedFacet, customTrigger);

	}

	public void add(IPolicyEvaluator policy, int waitPeriod,
			IManagedFacet managedFacet)
			throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException, NoEffectorException {
		addPolicy(policy, managedFacet);

		IPeriodicalTrigger evaluator = new PeriodicalPolicyEvaluator(policy,
				waitPeriod);

		registerManagedFacet(managedFacet, evaluator);

		evaluator.start();
	}

	public void add(IEventGenerator eventGenerator,
			IManagedFacet managedFacet)
			throws DoubleEventTypeOccuranceException,
			NotRegisteredManagedFacetReference {

		if (dependencyDetection) {

			String eventType = null;
			int alreadyExisting = 0;
			Iterator<String> eventGeneratorsEventTypes = eventGenerator
					.queryEventTypes();

			while (eventGeneratorsEventTypes.hasNext()) {

				eventType = eventGeneratorsEventTypes.next();

				if (eventTypesRegister.contains(eventType)) {
					alreadyExisting = -1;
				} else {

					++alreadyExisting;
				}
			}

			if (alreadyExisting != -1) {

				registerManagedFacet(managedFacet, eventGenerator);
				registerEventTypes(eventGenerator);

				// Diagnostic.trace(DiagnosticLevel.FINAL, "double check
				// "+alreadyExisting+" "+eventGenerator);
			} else {
				// TODO information which event generator uses the event type
				// which causes the exception to be thrown.
				throw new DoubleEventTypeOccuranceException("Event type "
						+ eventType + " is already used, the eventGenerator "
						+ eventGenerator
						+ " could not be registered with the manager\n");

				// Diagnostic.trace(DiagnosticLevel.FINAL, "not added!!!
				// "+alreadyExisting+" "+eventGenerator);
			}

		} else {
			// no dependency detection - just add it
			registerManagedFacet(managedFacet, eventGenerator);
			registerEventTypes(eventGenerator);
		}

	}

	/**
	 * Adds a specified system adapter to the registry by adding it to the
	 * specified managedFacet (if such an managedFacet exists).
	 * 
	 * @param managedFacet
	 *            the managedFacet the system adapter is associated with
	 * @param systemAdapter
	 *            the systemadapter to add
	 * @throws NotRegisteredManagedFacetReference
	 *             gets thrown if the managedFacet does not exist.
	 * @throws DoubleEventTypeOccuranceException
	 */
	private void registerManagedFacet(IManagedFacet managedFacet,
			ISystemAdapter systemAdapter)
			throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException {

		if (systemAdapter instanceof IEventGenerator) {
			IEventGenerator eventGenerator = (IEventGenerator) systemAdapter;
			eventGenerator.setEventBus(managerEventBus);
			if (dependencyDetection) {

				String eventType = null;
				int alreadyExisting = 0;
				Iterator<String> eventGeneratorsEventTypes = eventGenerator
						.queryEventTypes();

				while (eventGeneratorsEventTypes.hasNext()) {

					eventType = eventGeneratorsEventTypes.next();

					if (eventTypesRegister.contains(eventType)) {
						alreadyExisting = -1;
					} else {

						++alreadyExisting;
					}
				}

				if (alreadyExisting != -1) {

					registerEventTypes(eventGenerator);

					// Diagnostic.trace(DiagnosticLevel.FINAL, "double check
					// "+alreadyExisting+" "+eventGenerator);
				} else {
					// TODO information which event generator uses the event
					// type
					// which causes the exception to be thrown.
					throw new DoubleEventTypeOccuranceException("Event type "
							+ eventType
							+ " is already used, the eventGenerator "
							+ eventGenerator
							+ " could not be registered with the manager\n");

					// Diagnostic.trace(DiagnosticLevel.FINAL, "not added!!!
					// "+alreadyExisting+" "+eventGenerator);
				}

			} else {
				// no dependency detection - just add it
				registerEventTypes(eventGenerator);
			}

		}

		// iteration is used instead of contains
		// because contains returns false if managedFacet is altered between to
		// checks
		// (may be because a hashSet changes the hashCode depending on it's
		// members)

		Iterator<IManagedFacet> alreadyRegisteredManagedFacet = managedFacetRegistry
				.iterator();
		boolean inIt = false;
		while (alreadyRegisteredManagedFacet.hasNext()) {

			IManagedFacet oneOfThem = alreadyRegisteredManagedFacet.next();
			if (oneOfThem.equals(managedFacet)) {
				inIt = true;
			}
		}
		if (!inIt) {
			throw new NotRegisteredManagedFacetReference(
					"managedFacet is not registered with the manager");
		}

		// managedFacet is registered -> lets add a reference to the
		// systemadapter to
		// it.

		managedFacet.addSystemAdapter(systemAdapter);

	}

	/**
	 * registers the event types generated by the event generator with the
	 * global manager event type register
	 * 
	 * @param eventGenerator
	 *            the eventGenerator to be added to the GAMF
	 */
	private void registerEventTypes(IEventGenerator eventGenerator) {
		// Diagnostic.trace(DiagnosticLevel.FINAL, "add "+eventGenerator);
		Iterator<String> eventGeneratorsEventTypes = eventGenerator
				.queryEventTypes();

		while (eventGeneratorsEventTypes.hasNext()) {
			String eventType = eventGeneratorsEventTypes.next();

			if (eventTypesRegister.add(eventType)) {
				// Diagnostic.trace("event type added" + eventType);
			}

		}

	}

	public IManagedFacet getManagedFacet(String managedFacetName) {

		Iterator<IManagedFacet> managedFacets = managedFacetRegistry.iterator();
		IManagedFacet managedFacetToReturn = null;

		while (managedFacets.hasNext()) {
			IManagedFacet managedFacet = managedFacets.next();
			if (managedFacet.getManagedFacetName().equalsIgnoreCase(
					managedFacetName)) {
				managedFacetToReturn = managedFacet;
			}
		}

		return managedFacetToReturn;
	}

	public boolean getDependencyDetection() {

		return this.dependencyDetection;
	}

	public IEventStore getEventStore() {

		return GenericAutonomicManagementFramework.eventStore;
	}

	public void removeManagedFacet(IManagedFacet managedFacet) {
		// TODO Auto-generated method stub

	}

	public void removeSystemAdapter(IManagedFacet managedFacet,
			Object systemAdapter) {
		// TODO Auto-generated method stub

	}

	public void setDependencyDetection(boolean dependency) {

		this.dependencyDetection = dependency;
	}

	public Set<IManagedFacet> showAllManagedFacet() {
		return managedFacetRegistry;
	}

	public String getEventGeneratorName() {
		// TODO Auto-generated method stub
		return null;
	}

	public String getEventGeneratorType() {
		// TODO Auto-generated method stub
		return null;
	}

	public Iterator<String> queryEventTypes() {
		// TODO Auto-generated method stub
		return null;
	}

	public void setEventBus(IEventBus eventBus) {
		managerEventBus = eventBus;

	}

	public Set<String> getEventTypesRegister() {

		return GenericAutonomicManagementFramework.eventTypesRegister;
	}

	public IMetricValueStore getMetricResultStore() {
		return metricResultStore;
	}

	
	
	/**
	 * only needs to be implemented if a higher level manager manages this one. 
	 */
	public String getSystemAdapterCategory() {
		return null;
	}
	
	/**
	 * only needs to be implemented if a higher level manager manages this one. 
	 */
	public String getSystemAdapterIdentifier() {
		return null;
	}
	
	/**
	 * only needs to be implemented if a higher level manager manages this one. 
	 */
	public void triggerEventGeneration(String eventType,
			Map<String, String> msgs) throws EventGenerationException {
	}

	
	/**
	 * only needs to be implemented if a higher level manager manages this one. 
	 */
	public void triggerEventGeneration(String eventType, String msgs) throws EventGenerationException {	
	}

	
	/**
	 * only needs to be implemented if a higher level manager manages this one. 
	 */
	public void triggerEventGeneration(String eventType) throws EventGenerationException {
	}

	

}
