/*
 *  ASA Library
 *  Copyright (C) 2004-2008 Distributed Systems Architecture Research Group
 *  http://asa.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package gamf.interfaces.framework;

import java.util.Set;

import gamf.framework.exceptions.DoubleManagedFacetNameOccurenceException;
import gamf.framework.exceptions.DoubleEventTypeOccuranceException;
import gamf.framework.exceptions.NoEffectorException;
import gamf.framework.exceptions.NotRegisteredManagedFacetReference;
import gamf.interfaces.systemAdapter.IEffector;
import gamf.interfaces.systemAdapter.IEventGenerator;
import gamf.interfaces.systemAdapter.IMetricExtractor;
import gamf.interfaces.systemAdapter.IPolicyEvaluator;

/**
 * 
 * Provides access to the framework and describes a generic autonomic manager.
 * <p>
 * System adapter components can be registered, configured and categorised through this interface. Further to that the
 * interface allows the configuration of the framework's behaviour. When a system adapter component is registered it
 * must also be registered with an managedFacet, therefore an managedFacet must be defined before a component is registered.
 * </p>
 * <p>
 * In the case managers are nested, a manager provides an event generator interface for a higher level manager.
 * </p>
 * 
 * @author markus
 * 
 */
public abstract interface IGAMF extends IEventGenerator {

	/**
	 * Sets the dependency detection, to configure the dependency detection framework.
	 * <p>
	 * If the dependency detection is true the framework checks, during insertion time, if an inserted system adapter
	 * component is used already for another managedFacet. If it is used for managing another managedFacet, the programmer is
	 * informed about that. If dependency detection is true is also informs if an event type is already registered or
	 * not at insertion time.
	 * </p>
	 * 
	 * @param dependency true if dependency detection should be enabled
	 * 
	 */
	void setDependencyDetection(boolean dependency);

	/**
	 * Determines whether dependency detection is set.
	 * 
	 * @return true if dependency detection is set
	 */
	boolean getDependencyDetection();

	/**
	 * Adds an autonomic controlled managedFacet which allows system adapters to be associated with.
	 * <p>
	 * Every component of an managedFacet (system adapter) needs to be registered and categorised through managedFacets.
	 * </p>
	 * 
	 * @param managedFacetName describes an managedFacet that is controlled autonomically and contains all it's system adapter
	 *            components
	 * @return TODO
	 * @throws DoubleManagedFacetNameOccurenceException
	 */

	IManagedFacet registerManagedFacet(String managedFacetName) throws DoubleManagedFacetNameOccurenceException;

	/**
	 * Returns the managedFacet with the name managedFacetName.
	 * <p>
	 * The managedFacet must be added to the framework before any system adapter can be associated with this managedFacet.
	 * </p>
	 * 
	 * @param managedFacetName the managedFacets name
	 * @return the managedFacet IManagedFacet
	 */

	IManagedFacet getManagedFacet(String managedFacetName);

	/**
	 * Adds an effector to the framework and associates it with the specified managedFacet.
	 * <p>
	 * Through an effector a parameter of an unmanaged system can be controlled. An effector can also be used to allow
	 * a higher level manager to control parameters of a lower level manager which are directly influencing the lower
	 * level manager's policy objectives. The effector can be requested (like all categorised system adapters) through
	 * IManagedFacet. To allow multiple usage, an effector contains a description of what it does.
	 * </p>
	 * 
	 * @param effector for controlling parameters of a system
	 * @param managedFacet a systems autonomic managedFacet that is controlled
	 * @throws NotRegisteredManagedFacetReference
	 * @throws DoubleEventTypeOccuranceException
	 */

	void add(IEffector effector, IManagedFacet managedFacet) throws NotRegisteredManagedFacetReference, DoubleEventTypeOccuranceException;

	/**
	 * Adds an "event triggered" metric to the framework and associates it with the specified managedFacet.
	 * 
	 * 
	 * @param metric the metric to evaluate
	 * @param trigger invokes the evaluation of the metric on a specified condition, the trigger may be a custom
	 *            trigger provided by the system adapter programmer
	 * @param managedFacet a systems autonomic managedFacet that is controlled
	 * @throws NotRegisteredManagedFacetReference
	 * @throws DoubleEventTypeOccuranceException
	 */
	void add(IMetricExtractor metric, IEventTrigger trigger, IManagedFacet managedFacet) throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException;

	/**
	 * Adds an "event triggered" metric to the framework and associates it with the specified managedFacet.
	 * 
	 * 
	 * @param metric the metric to evaluate
	 * @param eventNameToBeTriggerendOn makes the manager creating a (standard) trigger to trigger the metric
	 *            evaluation on the specified eventTypeName
	 * @param managedFacet a systems autonomic managedFacet that is controlled
	 * @throws NotRegisteredManagedFacetReference
	 * @throws DoubleEventTypeOccuranceException
	 */

	void add(IMetricExtractor metric, String eventNameToBeTriggerendOn, IManagedFacet managedFacet) throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException;

	/**
	 * Adds a metric to the manager which is not evaluated by the manager. Important: If such the specified metric
	 * should ever be evaluated the system adapter developer needs to make sure that that will happen.
	 * 
	 * @param metric the metric which is never called from an internal manager component 
	 * @param managedFacet a systems autonomic managedFacet this component is used for 
	 * @throws NotRegisteredManagedFacetReference
	 * @throws DoubleEventTypeOccuranceException
	 */

	void add(IMetricExtractor metric, IManagedFacet managedFacet) throws NotRegisteredManagedFacetReference, DoubleEventTypeOccuranceException;

	/**
	 * Adds a "regularly invoked" metric to the framework and associates it with the specified managedFacet.
	 * 
	 * 
	 * @param metric the metric to evaluate
	 * @param waitPeriod the interval in milliseconds between successive evaluations
	 * @param managedFacet a system's autonomic managedFacet that is controlled
	 * @throws NotRegisteredManagedFacetReference
	 * @throws DoubleEventTypeOccuranceException
	 */

	void add(IMetricExtractor metric, int waitPeriod, IManagedFacet managedFacet) throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException;

	/**
	 * Adds an "event triggered" policy to the framework and associates it with the specified managedFacet.
	 * 
	 * 
	 * @param policy the policy to evaluate
	 * @param trigger invokes the evaluation of the policy on a specified condition, the trigger may be a custom
	 *            trigger provided by the system adapter programmer
	 * @param managedFacet a systems autonomic managedFacet that is controlled
	 * @throws NotRegisteredManagedFacetReference
	 * @throws DoubleEventTypeOccuranceException
	 */

	void add(IPolicyEvaluator policy, IEventTrigger trigger, IManagedFacet managedFacet) throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException;

	/**
	 * Adds an "event triggered" policy to the framework and associates it with the specified managedFacet.
	 * 
	 * @param policy policy the policy to evaluate
	 * 
	 * @param eventNameToBeTriggerendOn causes the manager to create a (standart) trigger
	 * 
	 * @param managedFacet a systems autonomic managedFacet that is controlled
	 * @throws NotRegisteredManagedFacetReference
	 * @throws DoubleEventTypeOccuranceException
	 * @throws NoEffectorException
	 */

	void add(IPolicyEvaluator policy, String eventNameToBeTriggerendOn, IManagedFacet managedFacet) throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException, NoEffectorException;

	/**
	 * Adds a "regularly invoked" policy to the framework and associates it with the specified managedFacet.
	 * 
	 * 
	 * @param policy the metric to evaluate
	 * @param waitPeriod the interval in milliseconds between successive evaluations
	 * @param managedFacet a system's autonomic managedFacet that is controlled
	 * @throws NotRegisteredManagedFacetReference
	 * @throws DoubleEventTypeOccuranceException
	 * @throws NoEffectorException
	 */

	void add(IPolicyEvaluator policy, int waitPeriod, IManagedFacet managedFacet) throws NotRegisteredManagedFacetReference,
			DoubleEventTypeOccuranceException, NoEffectorException;

	/**
	 * Adds an event generator to the framework and associates it with the specified managedFacet.
	 * 
	 * @param eventGenerator the event generator to add
	 * 
	 * @param managedFacet a system's autonomic managedFacet that is controlled
	 * @throws DoubleEventTypeOccuranceException
	 * @throws NotRegisteredManagedFacetReference
	 */
	void add(IEventGenerator eventGenerator, IManagedFacet managedFacet) throws DoubleEventTypeOccuranceException,
			NotRegisteredManagedFacetReference;

	/**
	 * Returns all managedFacets currently managed by the framework
	 * 
	 * @return Set of currently managed managedFacets
	 */

	Set<IManagedFacet> showAllManagedFacet();

	/**
	 * Removes everything that is associated with an managedFacet from the framework.
	 * 
	 * @param managedFacet which should be removed along all its system adapters
	 * 
	 */

	void removeManagedFacet(IManagedFacet managedFacet);

	/**
	 * Removes an individual part of the managedFacet from the framework.
	 * 
	 * @param managedFacet the managedFacet the specified system adapter is used for
	 * @param systemAdapter the part of the managedFacet that is removed from the framework
	 */

	void removeSystemAdapter(IManagedFacet managedFacet, Object systemAdapter);

	/**
	 * Returns the event store
	 * 
	 * @return eventstore used by gamf
	 */

	IEventStore getEventStore();

	/**
	 * returns the event types register
	 * 
	 * @return the event types register
	 * 
	 */

	Set<String> getEventTypesRegister();

	/**
	 * returns the metric result store
	 * 
	 * @return the metric result store
	 */

	IMetricValueStore getMetricResultStore();

}
