/*
 *  ASA Library
 *  Copyright (C) 2004-2008 Distributed Systems Architecture Research Group
 *  http://asa.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package gamf.interfaces.systemAdapter;

import java.util.Set;

/**
 * Provides a register to list and add available "provided" system adapters for the system.
 *  
 * <p>
 * If a system is designed with autonomic management and the use of the generic autonomic manager in mind, 
 * a system programmer can provide effectors and event generators and allow a system adapter programmer to discover these through this interface.
 * This interface must be integrated into the system's public interfaces.  
 * </p>
 * @author markus
 *
 */
public interface IManaged {
	/**
	 * Adds an event generator to the registry. 
	 * 
	 * @param eventGenerator the event generator to add
	 */

	void addEventGenerator(IEventGenerator eventGenerator);
	/**
	 * Adds an effector to the registry. 
	 * 
	 * @param effector the effector to add. 
	 */
	
	void addEffector(IEffector effector);
	
	/**
	 * Returns all registered effectors.
	 * 
	 * @return all registered effectors
	 */
	
	Set<IEffector> getAllEffectors();
	
	/**
	 * Returns all registered event generators.
	 * 
	 * @return all registered event generators.
	 */
	
	Set<IEventGenerator> getAllEventGenerators();
	
	/**
	 * Returns all registered effectors with the specified effector name.
	 * 
	 * @param effectorName the name of the effector(s) to look for
	 * 
	 * @return all registered effectors with name effectorName
	 * 
	 */
	
	Set<IEffector> getEffectorByName(String effectorName);
	
	
	/**
	 * Returns all registered event generators with the specified event generator name.
	 * 
	 * @param eventGeneratorName the name of the event generator(s) to look for
	 * 
	 * @return all registered event generators with name event generator
	 * 
	 */
	
	Set<IEventGenerator> getEventGeneratorByName(String eventGeneratorName);
	
	
	/**
	 * Returns all registered effectors with the specified effector type.
	 * 
	 * @param effectorType the type of the effector(s) to look for
	 * 
	 * @return all registered effectors of type effectorType
	 * 
	 */	
	
	Set<IEffector> getEffectorByType(String effectorType);

	
	/**
	 * Returns all registered event generators with the specified event generator type.
	 * 
	 * @param eventGeneratorType the type of the event generator(s) to look for
	 * 
	 * @return all registered event generators of type eventGeneratorType
	 * 
	 */	
	
	Set<IEventGenerator> getEventGeneratorByType(String eventGeneratorType);
	
	
	/**
	 * Returns all registered effectors with the specified effector name and the specified effector type.
	 * 
	 * @param effectorName the name of the effector(s) to look for
	 * @param effectorType the type of the effector(s) to look for 
	 * 
	 * @return all registered effectors with name effectorName and type effectorType
	 * 
	 */
	
	Set<IEffector> getEffector(String effectorName,String effectorType);
	
	
	/**
	 * Returns all registered effectors with the specified effector name and the specified effector type.
	 * 
	 * 
	 * @param eventGeneratorName the name of the event generator(s) to look for
	 * @param eventGeneratorType the type of the event generator(s) to look for
	 *  
	 * @return all registered effectors with name effectorName and type effectorType
	 * 
	 */
	
	Set<IEventGenerator> getEventGenerator(String eventGeneratorName,String eventGeneratorType);

	
}
